'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _cssClassCombiner = require('css-class-combiner');

var _cssClassCombiner2 = _interopRequireDefault(_cssClassCombiner);

var _AbstractSelection2 = require('./AbstractSelection');

var _AbstractSelection3 = _interopRequireDefault(_AbstractSelection2);

var _calculate = require('../core/calculate');

var _events = require('../utils/events');

var _cursors = require('../utils/cursors');

var _cursors2 = _interopRequireDefault(_cursors);

var _sides = require('../utils/sides');

var sides = _interopRequireWildcard(_sides);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MIN_DIMENSION_SIZE_FOR_BIG_HANDLE = 40;
var BIG_HANDLE_SIZE = 8;
var SMALL_HANDLE_SIZE = 5;

var getHandleSize = function getHandleSize(regionDimensions) {
  var isRegionTooSmall = Object.values(regionDimensions).some(function (d) {
    return d < MIN_DIMENSION_SIZE_FOR_BIG_HANDLE;
  });

  if (isRegionTooSmall) {
    return SMALL_HANDLE_SIZE;
  }

  return BIG_HANDLE_SIZE;
};

var InteractiveSelection = function (_AbstractSelection) {
  _inherits(InteractiveSelection, _AbstractSelection);

  function InteractiveSelection(props) {
    var _this$handlers;

    _classCallCheck(this, InteractiveSelection);

    var _this = _possibleConstructorReturn(this, (InteractiveSelection.__proto__ || Object.getPrototypeOf(InteractiveSelection)).call(this, props));

    _this.containerParameters = props.containerParameters;

    _this.state = {
      isFocused: false,
      isHovered: false,
      area: props.area,
      draggableMode: false
    };

    _this.handlers = (_this$handlers = {}, _defineProperty(_this$handlers, sides.TOP, _calculate.calculateTopSideResize), _defineProperty(_this$handlers, sides.BOTTOM, _calculate.calculateBottomSideResize), _defineProperty(_this$handlers, sides.RIGHT, _calculate.calculateRightSideResize), _defineProperty(_this$handlers, sides.LEFT, _calculate.calculateLeftSideResize), _defineProperty(_this$handlers, sides.BOTTOM_RIGHT, _calculate.calculateBottomRightSidesResize), _defineProperty(_this$handlers, sides.BOTTOM_LEFT, _calculate.calculateBottomLeftSidesResize), _defineProperty(_this$handlers, sides.TOP_RIGHT, _calculate.calculateTopRightSidesResize), _defineProperty(_this$handlers, sides.TOP_LEFT, _calculate.calculateTopLeftSidesResize), _this$handlers);

    _this.handleMouseDownOnSelection = _this.handleMouseDownOnSelection.bind(_this);
    _this.handleMouseUp = _this.handleMouseUp.bind(_this);
    _this.dragSelection = _this.dragSelection.bind(_this);
    _this.resizeSelection = _this.resizeSelection.bind(_this);
    return _this;
  }

  _createClass(InteractiveSelection, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.selectionEl.addEventListener('mousedown', this.handleMouseDownOnSelection);

      if (this.props.createdByUser) {
        this.manuallyStartToResize();
      }
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var area = nextProps.area,
          containerParameters = nextProps.containerParameters;


      this.containerParameters = containerParameters;

      this.setState({ area: area });
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.selectionEl.removeEventListener('mousedown', this.handleMouseDownOnSelection);
    }
  }, {
    key: 'setDefaultInnerOffset',
    value: function setDefaultInnerOffset() {
      var dimensions = this.state.area.dimensions;

      var innerOffsets = {
        left: dimensions.width / 2,
        top: dimensions.height / 2
      };

      this.setState({ innerOffsets: innerOffsets });
    }
  }, {
    key: 'getSelectionOffsets',
    value: function getSelectionOffsets() {
      var _selectionEl$getBound = this.selectionEl.getBoundingClientRect(),
          left = _selectionEl$getBound.left,
          top = _selectionEl$getBound.top;

      left += window.pageXOffset;
      top += window.pageYOffset;

      return {
        left: left, top: top
      };
    }
  }, {
    key: 'getInnerOffsets',
    value: function getInnerOffsets(event) {
      var clientX = event.clientX;

      var clientY = (0, _events.getClientY)(event);

      var _getSelectionOffsets = this.getSelectionOffsets(),
          selectionTopOffset = _getSelectionOffsets.top,
          selectionLeftOffset = _getSelectionOffsets.left;

      var innerOffsetLeft = clientX - selectionLeftOffset;
      var innerOffsetTop = clientY - selectionTopOffset;

      return {
        left: innerOffsetLeft,
        top: innerOffsetTop
      };
    }
  }, {
    key: 'handleMouseUp',
    value: function handleMouseUp() {
      var draggableMode = this.state.draggableMode;


      if (draggableMode) {
        this.stopDragSelection();
      } else {
        this.stopResizeSelection();
      }

      window.removeEventListener('mouseup', this.handleMouseUp);
    }
  }, {
    key: 'handleMouseDownOnSelection',
    value: function handleMouseDownOnSelection(event) {
      event.stopPropagation();
      this.selectionEl.focus();
      var target = event.target;


      if (this.props.freeze) {
        return;
      }

      var isMouseDownOnSelection = target === this.selectionEl;

      if (isMouseDownOnSelection) {
        this.startDrag(event);
      } else {
        var resizeSide = target.dataset.side;
        this.startResize(resizeSide);
      }

      window.addEventListener('mouseup', this.handleMouseUp);
    }
  }, {
    key: 'manuallyStartToResize',
    value: function manuallyStartToResize() {
      this.startResize(sides.BOTTOM_RIGHT);
      window.addEventListener('mouseup', this.handleMouseUp);
    }
  }, {
    key: 'dragSelection',
    value: function dragSelection(event) {
      event.stopPropagation();
      var _state = this.state,
          area = _state.area,
          innerOffsets = _state.innerOffsets;

      var coordinates = (0, _calculate.calculateDragSelection)(event, area, innerOffsets, this.containerParameters);

      this.setState({ area: _extends({}, area, { coordinates: coordinates }) });
    }
  }, {
    key: 'resizeSelection',
    value: function resizeSelection(event) {
      event.stopPropagation();
      var calculate = this.handlers[this.resizeSide];
      var area = calculate(event, this.state.area, this.containerParameters);

      this.setState({
        area: _extends({}, this.state.area, area)
      });
    }
  }, {
    key: 'stopDragSelection',
    value: function stopDragSelection() {
      var isSingle = this.props.isSingle;


      this.setState({ draggableMode: false });
      this.props.onAreaUpdate(this.state.area);

      (0, _cursors.setCursor)(isSingle ? _cursors2.default.GRAB : _cursors2.default.DEFAULT);
      window.removeEventListener('mousemove', this.dragSelection);
    }
  }, {
    key: 'stopResizeSelection',
    value: function stopResizeSelection() {
      window.removeEventListener('mousemove', this.resizeSelection);
      this.props.onAreaUpdate(this.state.area);
    }
  }, {
    key: 'startResize',
    value: function startResize(resizeSide) {
      this.resizeSide = resizeSide;
      window.addEventListener('mousemove', this.resizeSelection);
    }
  }, {
    key: 'startDrag',
    value: function startDrag(event) {
      this.setState({
        draggableMode: true,
        innerOffsets: this.getInnerOffsets(event)
      });

      (0, _cursors.setCursor)(_cursors2.default.GRABBING);

      window.addEventListener('mousemove', this.dragSelection);
    }
  }, {
    key: 'getClassName',
    value: function getClassName() {
      var _state2 = this.state,
          isHovered = _state2.isHovered,
          isFocused = _state2.isFocused,
          draggableMode = _state2.draggableMode;
      var _props = this.props,
          className = _props.className,
          hoverClassName = _props.hoverClassName,
          focusClassName = _props.focusClassName,
          freeze = _props.freeze;


      return new _cssClassCombiner2.default('mr-selection').combine('mr-interactive-selection').combine(className).combineIf(isHovered, hoverClassName).combineIf(isFocused, focusClassName).combineIf(freeze, 'mr-selection--transparent').combineIf(draggableMode, 'mr-interactive-selection--drag');
    }
  }, {
    key: 'getStyles',
    value: function getStyles() {
      return _extends({}, this.props.style, this.getPositionStyles(), {
        '--handle-size': getHandleSize(this.state.area.dimensions) + 'px'
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var styles = this.getStyles();
      var className = this.getClassName();

      return _react2.default.createElement(
        'div',
        {
          tabIndex: 0,
          role: 'button',
          ref: function ref(el) {
            return _this2.selectionEl = el;
          },
          className: className,
          style: styles,
          onFocus: this.handleFocus,
          onBlur: this.handleBlur,
          onMouseOver: this.handleMouseOver,
          onMouseOut: this.handleMouseOut,
          onDragStart: function onDragStart(e) {
            return e.preventDefault();
          }
        },
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--top-left',
          'data-side': sides.TOP_LEFT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--top-center',
          'data-side': sides.TOP
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--top-right',
          'data-side': sides.TOP_RIGHT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--right-center',
          'data-side': sides.RIGHT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--bottom-right',
          'data-side': sides.BOTTOM_RIGHT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--bottom-center',
          'data-side': sides.BOTTOM
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--bottom-left',
          'data-side': sides.BOTTOM_LEFT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--left-center',
          'data-side': sides.LEFT
        })
      );
    }
  }]);

  return InteractiveSelection;
}(_AbstractSelection3.default);

InteractiveSelection.propTypes = {
  freeze: _propTypes2.default.bool,
  createdByUser: _propTypes2.default.bool,
  isSingle: _propTypes2.default.bool,
  onAreaUpdate: _propTypes2.default.func
};

InteractiveSelection.defaultProps = {
  freeze: false,
  createdByUser: false,
  isSingle: false,
  onAreaUpdate: null
};

exports.default = InteractiveSelection;