'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculateDragSelection = exports.calculateTopRightSidesResize = exports.calculateTopLeftSidesResize = exports.calculateBottomRightSidesResize = exports.calculateBottomLeftSidesResize = exports.calculateTopSideResize = exports.calculateBottomSideResize = exports.calculateLeftSideResize = exports.calculateRightSideResize = undefined;

var _events = require('../utils/events');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var replaceSecondWith = function replaceSecondWith(nextArg) {
  return function (arg, i) {
    if (i !== 1) return arg;

    return nextArg;
  };
};

var calculateRightSideResize = exports.calculateRightSideResize = function calculateRightSideResize() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var event = args[0],
      selectionParameters = args[1],
      containerParameters = args[2];
  var _selectionParameters$ = selectionParameters.coordinates,
      y = _selectionParameters$.y,
      x = _selectionParameters$.x,
      _selectionParameters$2 = selectionParameters.dimensions,
      width = _selectionParameters$2.width,
      height = _selectionParameters$2.height;
  var containerLeftBorder = containerParameters.offsets.left,
      containerWidth = containerParameters.dimensions.width,
      _containerParameters$ = containerParameters.extraPadding,
      extraPadding = _containerParameters$ === undefined ? { left: 0 } : _containerParameters$;


  var innerX = x - extraPadding.left;
  var clientX = event.clientX;

  var boxRightBorder = innerX + width;
  var eventXWithinContainer = clientX - containerLeftBorder;
  var differenceBetweenOldBorder = eventXWithinContainer - boxRightBorder;
  var nextWidth = void 0;

  var tooCloseToLeft = eventXWithinContainer - innerX <= 10;

  if (tooCloseToLeft) {
    return selectionParameters;
  }

  if (eventXWithinContainer > containerWidth) {
    nextWidth = containerWidth - innerX;
  } else {
    nextWidth = width + differenceBetweenOldBorder;
  }

  return Object.freeze({
    dimensions: { width: nextWidth, height: height },
    coordinates: { x: x, y: y }
  });
};

var calculateLeftSideResize = exports.calculateLeftSideResize = function calculateLeftSideResize() {
  for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args[_key2] = arguments[_key2];
  }

  var event = args[0],
      selectionParameters = args[1],
      containerParameters = args[2];
  var _selectionParameters$3 = selectionParameters.coordinates,
      y = _selectionParameters$3.y,
      x = _selectionParameters$3.x,
      _selectionParameters$4 = selectionParameters.dimensions,
      width = _selectionParameters$4.width,
      height = _selectionParameters$4.height;
  var containerLeftBorder = containerParameters.offsets.left,
      _containerParameters$2 = containerParameters.extraPadding,
      extraPadding = _containerParameters$2 === undefined ? { left: 0 } : _containerParameters$2;


  var innerX = x - extraPadding.left;
  var clientX = event.clientX;

  var boxRightBorder = innerX + width;
  var eventXWithinContainer = clientX - containerLeftBorder;
  var differenceBetweenOldBorder = eventXWithinContainer - innerX;
  var nextX = void 0;
  var nextWidth = void 0;

  var tooCloseToRight = boxRightBorder - eventXWithinContainer <= 10;

  if (tooCloseToRight) {
    return selectionParameters;
  }

  if (eventXWithinContainer <= 0) {
    nextX = 0;
    nextWidth = boxRightBorder;
  } else {
    nextX = innerX + differenceBetweenOldBorder;
    nextWidth = width - differenceBetweenOldBorder;
  }

  return Object.freeze({
    dimensions: { width: nextWidth, height: height },
    coordinates: { x: nextX + extraPadding.left, y: y }
  });
};

var calculateBottomSideResize = exports.calculateBottomSideResize = function calculateBottomSideResize() {
  for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    args[_key3] = arguments[_key3];
  }

  var event = args[0],
      selectionParameters = args[1],
      containerParameters = args[2];
  var _selectionParameters$5 = selectionParameters.coordinates,
      x = _selectionParameters$5.x,
      y = _selectionParameters$5.y,
      _selectionParameters$6 = selectionParameters.dimensions,
      width = _selectionParameters$6.width,
      height = _selectionParameters$6.height;
  var containerTopBorder = containerParameters.offsets.top,
      containerHeight = containerParameters.dimensions.height,
      _containerParameters$3 = containerParameters.extraPadding,
      extraPadding = _containerParameters$3 === undefined ? { top: 0 } : _containerParameters$3;


  var innerY = y - extraPadding.top;
  var clientY = (0, _events.getClientY)(event);
  var eventYWithinContainer = clientY - containerTopBorder;
  var boxBottomBorder = innerY + height;
  var differenceBetweenOldBorder = eventYWithinContainer - boxBottomBorder;
  var nextHeight = void 0;

  var tooCloseToTop = eventYWithinContainer - innerY <= 10;

  if (tooCloseToTop) {
    return selectionParameters;
  }

  if (eventYWithinContainer > containerHeight) {
    nextHeight = containerHeight - innerY;
  } else {
    nextHeight = height + differenceBetweenOldBorder;
  }

  return Object.freeze({
    coordinates: { x: x, y: y },
    dimensions: { height: nextHeight, width: width }
  });
};

var calculateTopSideResize = exports.calculateTopSideResize = function calculateTopSideResize() {
  for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
    args[_key4] = arguments[_key4];
  }

  var event = args[0],
      selectionParameters = args[1],
      containerParameters = args[2];
  var _selectionParameters$7 = selectionParameters.coordinates,
      x = _selectionParameters$7.x,
      y = _selectionParameters$7.y,
      _selectionParameters$8 = selectionParameters.dimensions,
      width = _selectionParameters$8.width,
      height = _selectionParameters$8.height;
  var containerTopBorder = containerParameters.offsets.top,
      _containerParameters$4 = containerParameters.extraPadding,
      extraPadding = _containerParameters$4 === undefined ? { top: 0 } : _containerParameters$4;


  var innerY = y - extraPadding.top;
  var clientY = (0, _events.getClientY)(event);
  var selectionBottomBorder = innerY + height;
  var eventYWithinContainer = clientY - containerTopBorder;
  var differenceBetweenOldBorder = innerY - eventYWithinContainer;

  var nextHeight = void 0;
  var nextY = void 0;

  var tooCloseToBottom = selectionBottomBorder - eventYWithinContainer <= 10;

  if (tooCloseToBottom) {
    return selectionParameters;
  }

  if (eventYWithinContainer <= 0) {
    nextHeight = selectionBottomBorder;
    nextY = 0;
  } else {
    nextHeight = height + differenceBetweenOldBorder;
    nextY = innerY - differenceBetweenOldBorder;
  }

  return Object.freeze({
    coordinates: { x: x, y: nextY + extraPadding.top },
    dimensions: { width: width, height: nextHeight }
  });
};

var calculateBottomLeftSidesResize = exports.calculateBottomLeftSidesResize = function calculateBottomLeftSidesResize() {
  for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
    args[_key5] = arguments[_key5];
  }

  var nextSelectionParams = calculateBottomSideResize.apply(undefined, args);
  var nextArgs = args.map(replaceSecondWith(nextSelectionParams));

  return calculateLeftSideResize.apply(undefined, _toConsumableArray(nextArgs));
};

var calculateBottomRightSidesResize = exports.calculateBottomRightSidesResize = function calculateBottomRightSidesResize() {
  for (var _len6 = arguments.length, args = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
    args[_key6] = arguments[_key6];
  }

  var nextSelectionParams = calculateBottomSideResize.apply(undefined, args);
  var nextArgs = args.map(replaceSecondWith(nextSelectionParams));

  return calculateRightSideResize.apply(undefined, _toConsumableArray(nextArgs));
};

var calculateTopLeftSidesResize = exports.calculateTopLeftSidesResize = function calculateTopLeftSidesResize() {
  for (var _len7 = arguments.length, args = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
    args[_key7] = arguments[_key7];
  }

  var nextSelectionParams = calculateTopSideResize.apply(undefined, args);
  var nextArgs = args.map(replaceSecondWith(nextSelectionParams));

  return calculateLeftSideResize.apply(undefined, _toConsumableArray(nextArgs));
};

var calculateTopRightSidesResize = exports.calculateTopRightSidesResize = function calculateTopRightSidesResize() {
  for (var _len8 = arguments.length, args = Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
    args[_key8] = arguments[_key8];
  }

  var updatedSelectionParams = calculateTopSideResize.apply(undefined, args);
  var nextArgs = args.map(replaceSecondWith(updatedSelectionParams));

  return calculateRightSideResize.apply(undefined, _toConsumableArray(nextArgs));
};

var calculateDragSelection = exports.calculateDragSelection = function calculateDragSelection() {
  for (var _len9 = arguments.length, args = Array(_len9), _key9 = 0; _key9 < _len9; _key9++) {
    args[_key9] = arguments[_key9];
  }

  var event = args[0],
      selectionParameters = args[1],
      innerOffsets = args[2],
      containerParameters = args[3];
  var _selectionParameters$9 = selectionParameters.dimensions,
      height = _selectionParameters$9.height,
      width = _selectionParameters$9.width;
  var _containerParameters$5 = containerParameters.offsets,
      containerLeftBorder = _containerParameters$5.left,
      containerTopBorder = _containerParameters$5.top,
      _containerParameters$6 = containerParameters.dimensions,
      containerWidth = _containerParameters$6.width,
      containerHeight = _containerParameters$6.height,
      _containerParameters$7 = containerParameters.extraPadding,
      extraPadding = _containerParameters$7 === undefined ? { left: 0, top: 0 } : _containerParameters$7;
  var clientX = event.clientX;

  var clientY = (0, _events.getClientY)(event);

  var x = clientX - containerLeftBorder - innerOffsets.left;
  var y = clientY - containerTopBorder - innerOffsets.top;

  y = y < 0 ? 0 : y;
  x = x < 0 ? 0 : x;

  if (x + width > containerWidth) {
    x = containerWidth - width;
  }

  if (y + height > containerHeight) {
    y = containerHeight - height;
  }

  return Object.freeze({
    x: x + extraPadding.left,
    y: y + extraPadding.top
  });
};